// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#if !defined(IW_GAME_BOX2D_H_)
#define IW_GAME_BOX2D_H_

#include "s3e.h"
#include "IwGx.h"
#include "IwGeom.h"

#include "IwGameUtil.h"
#include "IwGameString.h"
#include "IwGameXoml.h"
#include "IwGameShapes.h"

// To disable Box2D support please comment out or undefine IW_GAME_BOX2D
#define	IW_GAME_BOX2D

#if defined(IW_GAME_BOX2D)

#include "Box2D\Box2D.h"


//
//
//
//
// CIwGameBox2dCcollisionCallback - Box2D scene collision callback
//
//
//
//
class CIwGameBox2dCcollisionCallback : public b2ContactListener
{
public:
	void				BeginContact(b2Contact* contact);		// Called when one body begins to collide with another
	void				EndContact(b2Contact* contact);			// Called when one body drops colliding with another
};

// Glopbally defined collision collback used to marshal cross actor collision calls
extern CIwGameBox2dCcollisionCallback	g_CIwGameBox2dCcollisionCallback;


//
//
//
//
// CIwGameBox2dCollidable - Represents a collection of begin and end contact collisions. Objects that are part of the physics system usually derive from this class
//
//
//
//
class CIwGameBox2dCollidable
{
public:
	typedef CIwList<CIwGameBox2dCollidable*>::iterator _CollisionIterator;
	_CollisionIterator		started_begin()		{ return CollisionsStarted.begin(); }
	_CollisionIterator		started_end()		{ return CollisionsStarted.end(); }
	_CollisionIterator		ended_begin()		{ return CollisionsEnded.begin(); }
	_CollisionIterator		ended_end()			{ return CollisionsEnded.end(); }
	_CollisionIterator		incontact_begin()	{ return CollisionContacts.begin(); }
	_CollisionIterator		incontact_end()		{ return CollisionContacts.end(); }

	// Properties
protected:
	CIwList<CIwGameBox2dCollidable*> CollisionsStarted;		// List of collidables that have just come into contact with this collidable
	CIwList<CIwGameBox2dCollidable*> CollisionsEnded;			// List of collidables that have just lost contact with this collidable
	CIwList<CIwGameBox2dCollidable*> CollisionContacts;		// List of collidables that this one is in collision with
public:
	virtual void	addCollisionStarted(CIwGameBox2dCollidable* with)
	{
		CollisionsStarted.push_back(with);
		CollisionContacts.push_back(with);
	}
	void			clearCollisionsStarted()			{ CollisionsStarted.clear(); }
	int				getCollisionsStartedCount()			{ return CollisionsStarted.size(); }
	virtual void	addCollisionEnded(CIwGameBox2dCollidable* with)
	{
		CollisionsEnded.push_back(with);
		for (_CollisionIterator it = incontact_begin(); it != incontact_end(); ++it)
		{
			if (*it == with)
			{
				CollisionContacts.erase(it);
				break;
			}
		}
	}
	void			clearCollisionsEnded()				{ CollisionsEnded.clear(); }
	int				getCollisionsEndedCount()			{ return CollisionsEnded.size(); }
	void			clearCollisionContacts()			{ CollisionContacts.clear(); }
	int				getCollisionContactsCount()			{ return CollisionContacts.size(); }

	void			clearAllCollisions()
	{
		CollisionsStarted.clear();
		CollisionsEnded.clear();
	}
	// End of properties

	virtual ~CIwGameBox2dCollidable()
	{
		clearAllCollisions();
		CollisionContacts.clear();
	}
};

//
//
//
//
// CIwGameBox2dMaterial - Box2D physics material
//
//
//
//
class CIwGameBox2dMaterial : public IIwGameXomlResource
{
public:

protected:
	// Properties
	b2BodyType			BodyType;			// Physical body type
	float				Density;			// Density of body
	float				Friction;			// Amount of friction to apply
	float				Restitution;		// Bounciness of the object
public:
	void				setBodyType(b2BodyType type)		{ BodyType = type; }
	b2BodyType			getBodyType() const					{ return BodyType; }
	void				setDensity(float density)			{ Density = density; }
	float				getDensity() const					{ return Density; }
	void				setFriction(float friction)			{ Friction = friction; }
	float				getFriction() const					{ return Friction; }
	void				setRestitution(float restitution)	{ Restitution = restitution; }
	float				getRestitution() const				{ return Restitution; }
	// Properties end

public:
	CIwGameBox2dMaterial() : IIwGameXomlResource(), Density(1.0f), Friction(1.0f), Restitution(0.1f)  { setClassType("Box2dMaterial"); }
	virtual ~CIwGameBox2dMaterial() {}

	// Implementation of IIwGameXomlResource interface
	bool				LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node);
};

//
//  CIwGameBox2dMaterialCreator - Creates an instance of an animation object
//
class CIwGameBox2dMaterialCreator : public IIwGameXomlClassCreator
{
public:
	CIwGameBox2dMaterialCreator()
	{
		setClassName("Box2dMaterial");
	}
	IIwGameXomlResource*	CreateInstance(IIwGameXomlResource* parent)	{ return new CIwGameBox2dMaterial(); }
};

//
//
//
//
// CIwGameBox2dWorld - Box2D physics world
//
//
//
//
class CIwGameBox2dWorld
{
public:

protected:
	// Properties
	b2World*				World;					// Physical world
	CIwFVec2				WorldScale;				// Scaling between physical and viusal worlds (set to 0, 0 to disable physics update)
	int						VelocityIterations;		// Number of internal iterations used when computing velocities
	int						PositionIterations;		// Number of internal iterations used when computing positions
	CIwFVec2				Gravity;				// Scene gravity
	float					TimeStep;				// Physics time step in seconds
public:
	void					setWorldScale(float x, float y)			{ WorldScale.x = x; WorldScale.y = y; }
	CIwFVec2				getWorldScale()							{ return WorldScale; }
	b2World*				getWorld()								{ return World; }
	void					setVelocityIterations(int count)		{ VelocityIterations = count; }
	int						getVelocityIterations() const			{ return VelocityIterations; }
	void					setPositionIterations(int count)		{ PositionIterations = count; }
	int						getPositionIterations() const			{ return PositionIterations; }
	void					setGravity(float x, float y);
	CIwFVec2				getGravity() const						{ return Gravity; }
	// Properties end

public:
	CIwGameBox2dWorld() : World(NULL), WorldScale(10.0f, 10.0f), VelocityIterations(6), PositionIterations(3), Gravity(0.0f, 15.0f) {}
	virtual ~CIwGameBox2dWorld();

	// World initialisation
	virtual void			InitWorld();

	// World update
	virtual void			UpdateWorld(float dt);

	// Box2D world to pixel conversions
	float			WorldToPixelX(float x)						{ return x * WorldScale.x; }
	float			WorldToPixelY(float y)						{ return y * WorldScale.y; }
	float			PixelToWorldX(float x)						{ return x / WorldScale.x; }
	float			PixelToWorldY(float y)						{ return y / WorldScale.y; }

};

//
//
//
//
// CIwGameBox2dBody - Box2D physics body
//
//
//
//
class CIwGameBox2dBody :  public CIwGameBox2dCollidable
{
public:

protected:
	// Properties
	CIwGameBox2dWorld*	World;					// World that this body is attached to
	b2Body*				Body;					// Physical body
	b2Fixture*			Fixture;				// The physical fixture (managed by Box2D)
	CIwGameBox2dMaterial* BodyMaterial;			// Physical body attributes that are applied to this actors body
	CIwGameShape*		BodyShape;				// Physical body shape
public:
	void				setWorld(CIwGameBox2dWorld* world)		{ World = world; }
	CIwGameBox2dWorld*	getWorld()								{ return World; }
	void				setBody(b2Body* body);					// Setting directly will destroy fixture
	b2Body*				getBody()								{ return Body; }
	void				setFixture(b2Fixture* fixture);
	b2Fixture*			getFixture()							{ return Fixture; }
	void				setBodyMaterial(CIwGameBox2dMaterial* mat);	
	CIwGameBox2dMaterial* getBodyMaterial()						{ return BodyMaterial; }
	void				setBodyShape(CIwGameShape* body_shape)	{ BodyShape = BodyShape; }
	CIwGameShape*		getBodyShape()							{ return BodyShape; }
	void				setAsSensor(bool enable)				{ if (Fixture != NULL) Fixture->SetSensor(enable); }
	void				setCollisionFlags(int category_flags, int mask_flags, int collision_group);
	int					getCollisionCategory() const;
	int					getCollisionMask() const;
	int					getCollisionGroup() const;
	// Properties end

public:
	CIwGameBox2dBody() : CIwGameBox2dCollidable(), World(NULL), Body(NULL), Fixture(NULL), BodyMaterial(NULL), BodyShape(NULL)			{}
	virtual ~CIwGameBox2dBody();

	// Initialise the physical body
	void				InitBody(CIwGameBox2dWorld* world, CIwGameShape* body_shape, CIwGameBox2dMaterial* body_mat, CIwFVec2* pos, float angle, float com_x, float com_y);

	void				ReleaseBody();

	// Force application
	void				ApplyForce(float force_x, float force_y, float world_pos_x, float world_pos_y);
	void				ApplyForceToCenter(float force_x, float force_y);
	void				ApplyTorque(float torque);
	void				ApplyLinearImpulse(float impulse_x, float impulse_y, float world_pos_x, float world_pos_y);
	void				ApplyAngularImpulse(float impulse);
};

#endif	// IW_GAME_BOX2D_H_


#endif // IW_GAME_BOX2D_H_
