﻿// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#include "IwGameActor.h"
#include "IwGameScene.h"
#include "IwGameCamera.h"
#include "IwGameXoml.h"
#include "IwGameActions.h"


//
//
//
// CIwGameActor implementation
//
//
//

CIwGameActor::~CIwGameActor()
{
	// Only delete an unmanaged timeline
	if (Timeline != NULL)
	{
		if (!Timeline->isManaged())
		{
			SAFE_DELETE(Timeline)
		}
	}

	// Delete visual if one was attached
	if (Visual != NULL)
		Scene->getSpriteManager()->removeSprite(Visual);

	// Clean up Box2D body object
#if defined(IW_GAME_BOX2D)
	SAFE_DELETE(Box2dBody)
#endif // IW_GAME_BOX2D_H_

	// Delete events manager
	SAFE_DELETE(EventsManager)

}

void CIwGameActor::setCollisionRect(CIwRect& rect)
{
	CollisionRect = rect;
	int dx = rect.w >> 1;
	int dy = rect.h >> 1;
	CollisionSize = dx * dx + dy * dy;
}

void CIwGameActor::setCollisionSize(int size)
{
	CollisionRect.x = -size >> 1;
	CollisionRect.y = -size >> 1;
	CollisionRect.w = size;
	CollisionRect.h = size;
}

void CIwGameActor::setPosition(float x, float y)
{
	Position.x = x; Position.y = y;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
	{
		CIwGameBox2dWorld* world = Scene->getBox2dWorld();
		if (world != NULL)
			body->SetTransform(b2Vec2(world->PixelToWorldX(x), world->PixelToWorldY(y)), body->GetAngle());
	}
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setAngle(float angle)
{
	Angle = angle;
#if defined(IW_GAME_BOX2D)
	Angle = -Angle;
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
	{
		body->SetTransform(body->GetPosition(), (Angle * 3.1415927f) / 180.0f);	// Degrees to radians
	}
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setVelocity(float x, float y)
{
	Velocity.x = x;  Velocity.y = y;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
	{
		body->SetLinearVelocity(b2Vec2(x, y));
	}
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setVelocityDamping(float x, float y)
{
	VelocityDamping.x = x;  VelocityDamping.y = y;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
		body->SetLinearDamping(x);
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setVelocityDamping(float damping)
{
	setVelocityDamping(damping, damping);
}

void CIwGameActor::setAngularVelocity(float velocity)
{
	AngularVelocity = velocity;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
		body->SetAngularVelocity(AngularVelocity);
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setAngularVelocityDamping(float damping)
{
	AngularVelocityDamping = damping;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
		body->SetAngularVelocity(AngularVelocityDamping);
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setActive(bool active)
{
	if (IsActive == active)
		return;
	IsActive = active;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
		body->SetActive(active);
#endif	// IW_GAME_BOX2D
}


void CIwGameActor::Init()
{
}

void CIwGameActor::Reset()
{
	IsActive = true;
	IsVisible = true;
	IsCollidable = false;
	IsTappable = true;
	WrapPosition = false;
	Type = 0;
	Position.x = 0;
	Position.y = 0;
	OriginalPosition.x = 0;
	OriginalPosition.y = 0;
	Scale = 1.0f;
	OriginalAngle = 0;
	Angle = 0;
	Velocity.x = 0;
	Velocity.y = 0;
	AngularVelocity = 0;
	VelocityDamping.x = 1.0f;
	VelocityDamping.y = 1.0f;
	AngularVelocityDamping = 1.0f;
	CollisionSize = 0;
	CollisionRect.x = 0;
	CollisionRect.y = 0;
	CollisionRect.w = 0;
	CollisionRect.h = 0;
	Colour.r = 0xff;
	Colour.g = 0xff;
	Colour.b = 0xff;
	Colour.a = 0xff;
	Layer = 0;
}

bool CIwGameActor::Update(float dt)
{
	if (!IsActive)
		return true;

#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
	{
		// Update actor position and angle to match the physics engine
		float x = body->GetPosition().x;
		float y = body->GetPosition().y;
		CIwGameBox2dWorld* world = Scene->getBox2dWorld();
		if (world != NULL)
		{
			Position.x = world->WorldToPixelX(x);
			Position.y = world->WorldToPixelY(y);
		}
		Angle = (body->GetAngle() * 180.0f) / 3.1415927f;	// Radians to degrees
	}
	else
#endif	// IW_GAME_BOX2D
	{
		// Update velocity
		PreviousPosition = Position;
		Position += Velocity * dt;
		Velocity.x *= VelocityDamping.x;
		Velocity.y *= VelocityDamping.y;
		PreviousAngle = Angle;
		Angle += AngularVelocity * dt;
		AngularVelocity *= AngularVelocityDamping;
	}

	// Wrap actor position with scene extents (if enabled)
	CIwRect extents = Scene->getExtents();
	if (WrapPosition)
	{
#if defined(IW_GAME_BOX2D)
		bool wrapped = false;
		if (Position.x < extents.x)
		{
			Position.x += extents.w;
			wrapped = true;
		}
		else
		if (Position.x >= (extents.x + extents.w))
		{
			Position.x -= extents.w;
			wrapped = true;
		}
		if (Position.y < extents.y)
		{
			Position.y += extents.h;
			wrapped = true;
		}
		else
		if (Position.y >= (extents.y + extents.h))
		{
			Position.y -= extents.h;
			wrapped = true;
		}
		if (wrapped && body != NULL)
		{
			CIwGameBox2dWorld* world = Scene->getBox2dWorld();
			if (world != NULL)
				body->SetTransform(b2Vec2(world->PixelToWorldX(Position.x), world->PixelToWorldX(Position.y)), body->GetAngle());
		}
#else
		if (Position.x < extents.x)
			Position.x += extents.w;
		else
		if (Position.x >= (extents.x + extents.w))
			Position.x -= extents.w;
		if (Position.y < extents.y)
			Position.y += extents.h;
		else
		if (Position.y >= (extents.y + extents.h))
			Position.y -= extents.h;
#endif	// IW_GAME_BOX2D
	}

	// Update the animation timeline
	if (Timeline != NULL)
 		Timeline->Update(dt, this);

	// If this actor is collidable then add it to the scenes potential collidables list
	if (IsCollidable)
		Scene->addCollideable(this);

	return true;
}

bool CIwGameActor::UpdateVisual()
{
	if (Visual == NULL)
		return false;

	if (!IsVisible || !IsActive)
	{
		Visual->setVisible(false);
		return false;
	}

	Visual->setVisible(true);

	if (Scene->getCamera() != NULL)
	{
		CIwFVec2 cam_pos = Scene->getCamera()->getPosition();
		// Transfor actor properties to the visual
		Visual->setPosAngScale(	(int)(Position.x - cam_pos.x), 
								(int)(Position.y - cam_pos.y), 
								(iwangle)((Angle * IW_ANGLE_2PI) / 360.0f), (iwfixed)(Scale * IW_GEOM_ONE));
	}
	else
	{
		// Transfor actor properties to the visual
		Visual->setPosAngScale(	(int)(Position.x), 
								(int)(Position.y), 
								(iwangle)((Angle * IW_ANGLE_2PI) / 360.0f), (iwfixed)(Scale * IW_GEOM_ONE));
	}

	// Multiply actor colour by scene colour
	CIwColour c = Colour;
	CIwColour c2 = Scene->getColour();
	c.r = (uint8)(((int)c.r * c2.r) / 255);
	c.g = (uint8)(((int)c.g * c2.g) / 255);
	c.b = (uint8)(((int)c.b * c2.b) / 255);
	c.a = (uint8)(((int)c.a * c2.a) / 255);
	Visual->setColour(c);

	// Set visible layer
	Visual->setLayer(Layer);

	return true;
}

void CIwGameActor::removeVisual()
{
	if (Visual != NULL)
	{
		Scene->getSpriteManager()->removeSprite(Visual);
		Visual = NULL;
	}
}

CIwGameActor* CIwGameActor::HitTest(int x, int y)
{
	if (Visual == NULL || !isTappable())
		return NULL;

	if (Visual->HitTest(x, y))
		return this;		// We return a CIwGameActor so we can return a different actor if this actor hosts child actors

	return NULL;
}

bool CIwGameActor::CheckCollision(CIwGameActor* other)
{
	// This is just about as simple as colision checks can go. We check to see if the distance between the centre of both objects is less than the sum of the radii of both
	// Note that we do not use a proper distance between two points check because that involves expensive square root calculations, but this works just the same
	// The only thing to be careful of is if your coordinate system is huge and your numbers have a large range, just ensure that the term for d = dx * dx + dy * dy does not fall 
	// out of integer range
	CIwFVec2 other_pos = other->getPosition();
	int		other_collision_size = other->getCollisionSize();
	int		dx = (int)(Position.x - other_pos.x);
	int		dy = (int)(Position.y - other_pos.y);
	int		d_max = (CollisionSize >> 1) + (other_collision_size >> 1);
	int		d = dx * dx + dy * dy;

	d_max *= d_max;
	if (d <= d_max)
		return true;

	return false;
}

void CIwGameActor::OnTapped(int x, int y)
{
	ProcessEventActions(CIwGameXomlNames::OnTapped_Hash);
}

void CIwGameActor::OnBeginTouch(int x, int y)
{
	ProcessEventActions(CIwGameXomlNames::OnBeginTouch_Hash);
	Colour.a = 0x80;
}

void CIwGameActor::OnEndTouch(int x, int y)
{
	ProcessEventActions(CIwGameXomlNames::OnEndTouch_Hash);
	Colour.a = 0xff;
}

void CIwGameActor::ProcessEventActions(unsigned int event_name)
{
	// Find the event
	CIwGameEvent* evt = EventsManager->findEvent(event_name);
	if (evt != NULL)
	{
		// Find actions list
		CIwGameActions* actions = (CIwGameActions*)Scene->getResourceManager()->findResource(evt->ActionsName, CIwGameXomlNames::Actions_Hash);
		if (actions != NULL)
		{
			actions->Execute(this);
		}
	}
}

bool CIwGameActor::UpdateFromAnimation(CIwGameAnimInstance *animation)
{
	unsigned int element_name = animation->getTargetPropertyHash();

	if (element_name == CIwGameXomlNames::Position_Hash)
	{
		CIwGameAnimFrameVec2* frame = (CIwGameAnimFrameVec2*)animation->getCurrentData();
		Position.x = frame->data.x;
		Position.y = frame->data.y;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Velocity_Hash)
	{
		CIwGameAnimFrameVec2* frame = (CIwGameAnimFrameVec2*)animation->getCurrentData();
		Velocity.x = frame->data.x;
		Velocity.y = frame->data.y;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Angle_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		Angle = frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::AngularVelocity_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		AngularVelocity = frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Scale_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		Scale = frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Colour_Hash || element_name == CIwGameXomlNames::Color_Hash)
	{
		CIwGameAnimFrameVec4* frame = (CIwGameAnimFrameVec4*)animation->getCurrentData();
		Colour.r = (uint8)frame->data.x;
		Colour.g = (uint8)frame->data.y;
		Colour.b = (uint8)frame->data.z;
		Colour.a = (uint8)frame->data.w;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Layer_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		Layer = (int)frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Visible_Hash)
	{
		CIwGameAnimFrameBool* frame = (CIwGameAnimFrameBool*)animation->getCurrentData();
		IsVisible = frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::HitTest_Hash)
	{
		CIwGameAnimFrameBool* frame = (CIwGameAnimFrameBool*)animation->getCurrentData();
		IsTappable = frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Timeline_Hash)
	{
		CIwGameAnimFrameString* frame = (CIwGameAnimFrameString*)animation->getCurrentData();
		if (!frame->data.IsEmpty())
		{
			CIwGameAnimTimeline* timeline = (CIwGameAnimTimeline*)Scene->getResourceManager()->findResource(frame->data.c_str(), CIwGameXomlNames::Timeline_Hash);
			if (timeline != NULL && timeline != Timeline)
			{
				setTimeline(timeline);
				timeline->setTargetElement(this);
			}
		}
		return true;
	}

	return false;
}

bool CIwGameActor::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	if (parent->getClassTypeHash() != CIwGameXomlNames::Scene_Hash)
	{
		CIwGameError::LogError("Error: XOML - Actor needs to be declared inside a scene");
		return false;
	}

	// Add the actor to the scene
	CIwGameScene*	scene = (CIwGameScene*)parent;
	scene->addActor(this);

	// Check and apply style
	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();
		if (name_hash == CIwGameXomlNames::Style_Hash)
		{
			if (!(*it)->GetValue().IsEmpty())
			{
				// Find the style
				CIwGameStyle* style = (CIwGameStyle*)scene->getResourceManager()->findResource((*it)->GetValue().c_str(), CIwGameXomlNames::Style_Hash);
				if (style == NULL)
				{
					CIwGameError::LogError("Error: XOML - Could not find named style in the scene or in the global resources - ", (*it)->GetValue().c_str());
					return false;
				}
				// Apply the style
				CIwGameXoml::ApplyStyle(style, node);
			}
		}
	}

	// Process Actor attributes
	CIwGameString	timeline_name, box2dmaterial_name, shape_name;
	CIwFVec2		com(0, 0);
	CIwFVec3		collision_flags(0, 0, 0);
	bool			sensor = false;
	bool			vd_set = false;
	bool			avd_set = false;
	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Name_Hash)
		{
			setName((*it)->GetValue().c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::Type_Hash)
		{
			setType((*it)->GetValueAsInt());
		}
		else
		if (name_hash == CIwGameXomlNames::Position_Hash)
		{
			if (!(*it)->GetValueAsPoint(Position))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::Position");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::Velocity_Hash)
		{
			if (!(*it)->GetValueAsPoint(Velocity))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::Velocity");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::VelocityDamping_Hash)
		{
			if (!(*it)->GetValueAsPoint(VelocityDamping))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::VelocityDamping");
			}
			else
				vd_set = true;
		}
		else
		if (name_hash == CIwGameXomlNames::Angle_Hash)
		{
			Angle = (*it)->GetValueAsFloat();
		}
		else
		if (name_hash == CIwGameXomlNames::AngularVelocity_Hash)
		{
			AngularVelocity = (*it)->GetValueAsFloat();
		}
		else
		if (name_hash == CIwGameXomlNames::AngularVelocityDamping_Hash)
		{
			AngularVelocityDamping = (*it)->GetValueAsFloat();
			avd_set = true;
		}
		else
		if (name_hash == CIwGameXomlNames::Scale_Hash)
		{
			Scale = (*it)->GetValueAsFloat();
		}
		else
		if (name_hash == CIwGameXomlNames::Colour_Hash || name_hash == CIwGameXomlNames::Color_Hash)
		{
			if (!(*it)->GetValueAsColour(Colour))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::Colour");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::Layer_Hash)
		{
			Layer = (*it)->GetValueAsInt();
		}
		else
		if (name_hash == CIwGameXomlNames::Active_Hash)
		{
			IsActive = (*it)->GetValueAsBool();
		}
		else
		if (name_hash == CIwGameXomlNames::Visible_Hash)
		{
			IsVisible = (*it)->GetValueAsBool();
		}
		else
		if (name_hash == CIwGameXomlNames::Collidable_Hash)
		{
			IsCollidable = (*it)->GetValueAsBool();
		}
		else
		if (name_hash == CIwGameXomlNames::CollisionSize_Hash)
		{
			setCollisionSize((*it)->GetValueAsBool());
		}
		else
		if (name_hash == CIwGameXomlNames::CollisionRect_Hash)
		{
			CIwRect rect;
			if (!(*it)->GetValueAsRect(rect))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::CollisionRect");
			}
			setCollisionRect(rect);
		}
		else
		if (name_hash == CIwGameXomlNames::WrapPosition_Hash)
		{
			WrapPosition = (*it)->GetValueAsBool();
		}
		else
		if (name_hash == CIwGameXomlNames::Timeline_Hash)
		{
			timeline_name = (*it)->GetValue();
		}
		else
		if (name_hash == CIwGameXomlNames::OnTapped_Hash)
		{
			EventsManager->addEvent("OnTapped", (*it)->GetValue().c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::OnBeginTouch_Hash)
		{
			EventsManager->addEvent("OnBeginTouch", (*it)->GetValue().c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::OnEndTouch_Hash)
		{
			EventsManager->addEvent("OnEndTouch", (*it)->GetValue().c_str());
		}
#if defined(IW_GAME_BOX2D)
		else
		if (name_hash == CIwGameXomlNames::Box2dMaterial_Hash)
		{
			box2dmaterial_name = (*it)->GetValue();
		}
		else
		if (name_hash == CIwGameXomlNames::Shape_Hash)
		{
			shape_name = (*it)->GetValue();
		}
		else
		if (name_hash == CIwGameXomlNames::COM_Hash)
		{
			if (!(*it)->GetValueAsPoint(com))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::COM");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::Sensor_Hash)
		{
			sensor = (*it)->GetValueAsBool();
		}
		else
		if (name_hash == CIwGameXomlNames::CollisionFlags_Hash)
		{
			if (!(*it)->GetValueAsPoint3(collision_flags))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::CollisionFlags");
			}
		}
#endif // IW_GAME_BOX2D
	}

	// Search the scenes resources for the specified timeline
	if (!timeline_name.IsEmpty())
	{
		CIwGameAnimTimeline* timeline = (CIwGameAnimTimeline*)scene->getResourceManager()->findResource(timeline_name.c_str(), CIwGameXomlNames::Timeline_Hash);
		if (timeline == NULL)
		{
			CIwGameError::LogError("Error: XOML - Could not find named timeline in the scene or in the global resources - ", timeline_name.c_str());
			return false;
		}
		setTimeline(timeline);
		timeline->setTargetElement(this);
	}

#if defined(IW_GAME_BOX2D)
	// Check for shape and set if present
	CIwGameShape* shape = NULL;
	if (!shape_name.IsEmpty())
	{
		shape = (CIwGameShape*)scene->getResourceManager()->findResource(shape_name.c_str(), CIwGameXomlNames::Shape_Hash);
		if (shape == NULL)
		{
			CIwGameError::LogError("Error: XOML - Could not find named shape in the scene or in the global resources - ", shape_name.c_str());
			return false;
		}
	}

	// Check for box2d material and set if present
	if (!box2dmaterial_name.IsEmpty())
	{
		CIwGameBox2dMaterial* material = (CIwGameBox2dMaterial*)scene->getResourceManager()->findResource(box2dmaterial_name.c_str(), CIwGameXomlNames::Box2dMaterial_Hash);
		if (material == NULL)
		{
			CIwGameError::LogError("Error: XOML - Could not find named material in the scene or in the global resources - ", box2dmaterial_name.c_str());
			return false;
		}

		if (shape == NULL)
		{
			CIwGameError::LogError("Warning: XOML - Cannot attach Box2d physics to an actor that has no shape");
		}
		else
		{
			Angle = -Angle;
			Box2dBody = new CIwGameBox2dBody();
			if (shape != NULL)
				Box2dBody->setBodyShape(shape);
			Box2dBody->InitBody(Scene->getBox2dWorld(), shape, material, &Position, Angle, com.x, com.y);
			if (!vd_set)
			{
				VelocityDamping.x = 0;
				VelocityDamping.y = 0;
			}
			if (!avd_set)
				AngularVelocityDamping = 0;
			setVelocity(Velocity.x, Velocity.y);
			setVelocityDamping(VelocityDamping.x, VelocityDamping.y);
			setAngularVelocity(AngularVelocity);
			setAngularVelocityDamping(AngularVelocityDamping);
			Box2dBody->setAsSensor(sensor);
			Box2dBody->setCollisionFlags((int)collision_flags.x, (int)collision_flags.y, (int)collision_flags.z);
		}
	}

#endif // IW_GAME_BOX2D

	// Process inner tags
	if (!IW_GAME_XOML->Process(this, node))
		return false;


	return true;
}

//
//
//
//
//
// CIwGameActorImage implementation
//
//
//
//
//
bool CIwGameActorImage::InitWithImage(CIwGameImage* image, int width, int height)
{
	// Create sprite
	if (image != NULL)
	{
		CIwGameBitmapSprite* sprite = new CIwGameBitmapSprite();
		if (sprite == NULL)
			return false;

		// Set sprite image
		sprite->setImage(image);
		sprite->setSrcDest(0, 0, width, height);

		// Set sprite as visual
		Visual = sprite;

		// Add sprite to the sprite manager so it can be managed and drawn
		Scene->getSpriteManager()->addSprite(sprite);
	}

	return true;
}

bool CIwGameActorImage::Init(CIwGameImage* image, int width, int height)
{
	CIwGameActor::Init();

	// Create sprite
	if (!InitWithImage(image, width, height))
		return false;

	return true;
}

bool CIwGameActorImage::UpdateVisual()
{
	if (CIwGameActor::UpdateVisual())
	{
	}

	return false;
}

bool CIwGameActorImage::UpdateFromAnimation(CIwGameAnimInstance *animation)
{
	if (CIwGameActor::UpdateFromAnimation(animation))
		return true;

	unsigned int element_name = animation->getTargetPropertyHash();

	if (element_name == CIwGameXomlNames::SrcRect_Hash)
	{
		CIwGameAnimFrameRect* frame = (CIwGameAnimFrameRect*)animation->getCurrentData();
		((CIwGameBitmapSprite*)Visual)->setSrcRect(&frame->data);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Size_Hash)
	{
		CIwGameAnimFrameVec2* frame = (CIwGameAnimFrameVec2*)animation->getCurrentData();
		((CIwGameBitmapSprite*)Visual)->setDestSize((int)frame->data.x, (int)frame->data.y);
		return true;
	}

	return false;
}

bool CIwGameActorImage::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	if (parent->getClassTypeHash() != CIwGameXomlNames::Scene_Hash)
	{
		CIwGameError::LogError("Error: XOML - Actor needs to be declared inside a scene type class");
		return false;
	}

	// Load main actor attributes
	if (!CIwGameActor::LoadFromXoml(parent, load_children, node))
		return false;

	// Process ActorImage specific attributes
	CIwGameScene*	scene = (CIwGameScene*)parent;
	CIwGameString	image_name;
	CIwFVec2		size;
	CIwRect			src_rect(0, 0, 0, 0);
	int				image_transform = IW_2D_IMAGE_TRANSFORM_NONE;

	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Image_Hash)
		{
			image_name = (*it)->GetValue();
		}
		else
		if (name_hash == CIwGameXomlNames::Size_Hash)
		{
			if (!(*it)->GetValueAsPoint(size))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for ActorImage::Size");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::SrcRect_Hash)
		{
			if (!(*it)->GetValueAsRect(src_rect))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for ActorImage::SrcRect");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::FlipX_Hash)
		{
			if ((*it)->GetValueAsBool())
				image_transform |= IW2D_FLIP_X_F;
		}
		else
		if (name_hash == CIwGameXomlNames::FlipY_Hash)
		{
			if ((*it)->GetValueAsBool())
				image_transform |= IW2D_FLIP_Y_F;
		}
	}

	// Search the scenes resources for the specified image
	CIwGameImage* image = (CIwGameImage*)scene->getResourceManager()->findResource(image_name.c_str(), CIwGameXomlNames::Image_Hash);
	if (image == NULL)
	{
		CIwGameError::LogError("Error: XOML - Could not find named image in the scene or in the global resources - ", image_name.c_str());
		return false;
	}

	// Initialise the image actor
	Scene = scene;
	Init(image, (int)size.x, (int)size.y);
	if (src_rect.w != 0)
	{
		CIwGameBitmapSprite* visual = (CIwGameBitmapSprite*)Visual;
		visual->setSrcRect(&src_rect);
		visual->setImageTransform((CIw2DImageTransform)image_transform);
	}

	return true;
}

//
//
//
//
//  CIwGameXomlAction_Actors implementation
//
//
//
//
void CIwGameXomlAction_Actors::Execute(IIwGameXomlResource* source, CIwGameAction* action)
{
	CIwGame* game = NULL;
	CIwGameScene* scene = NULL;
	CIwGameActor* actor = NULL;

	if (source->getClassTypeHash() == CIwGameXomlNames::Scene_Hash)
	{
		scene = (CIwGameScene*)source;
		game = scene->getParent();
	}
	else
	if (source->getClassTypeHash() == CIwGameXomlNames::Actor_Hash)
	{
		actor = (CIwGameActor*)source;
		scene = actor->getScene();
		game = scene->getParent();
	}

	CIwGameActor* new_actor = action->Parameter1.IsEmpty() ? actor : scene->findActor(action->Parameter1.getHash());
	if (new_actor != NULL)
	{
		switch (Type)
		{
		case Action_HideActor:
			new_actor->setVisible(false);
			break;
		case Action_ShowActor:
			new_actor->setVisible(true);
			break;
		case Action_ActivateActor:
			new_actor->setActive(true);
			break;
		case Action_DeactivateActor:
			new_actor->setActive(false);
			break;
		case Action_KillActor:
			scene->removeActor(new_actor);
			break;
		}
	}
	else
		CIwGameError::LogError("Warning: Actions - Actor was not found - ", action->Parameter1.c_str());
}

